package jp.sbcr.java_framework.validator;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;

import jp.sbcr.java_framework.validator.annotation.Property;
import jp.sbcr.java_framework.validator.annotation.PropertyName;
import jp.sbcr.java_framework.validator.annotation.Validator;
import jp.sbcr.java_framework.validator.annotation.Validators;
import jp.sbcr.java_framework.validator.exception.ValidateException;

import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.seasar.framework.util.StringUtil;

public abstract class JChecks {

	/**
	 * バリデーションを行います。 <br> {@link Validators}
	 * アノテーションが付いた、全てのプロパティーに対しバリデーションを行います。
	 * 
	 * @param bean
	 *            バリデーション対象のbean
	 * @param errors
	 *            エラーを追加するActionMessages
	 * @param request
	 * @return Beanにセットされた値が全て有効な値の場合はtrue。そうでない場合はfalse。
	 */
	public static boolean validate(Object bean,
			ActionMessages errors, HttpServletRequest request) {
		if (bean == null) {
			return true;
		}

		boolean ret = true;
		BeanInfo beanInfo = getBeanInfo(bean);
		PropertyDescriptor[] propertyDescriptors = beanInfo
				.getPropertyDescriptors();
		for (int i = 0; i < propertyDescriptors.length; i++) {
			PropertyDescriptor descriptor = propertyDescriptors[i];
			Method writeMethod = descriptor.getWriteMethod();
			if (writeMethod == null) {
				continue;
			}
			boolean valid = validate(bean, descriptor, errors,
					request);
			ret = ret == true ? valid : ret;
		}
		return ret;
	}

	private static BeanInfo getBeanInfo(Object bean) {
		try {
			return Introspector.getBeanInfo(bean.getClass());
		} catch (IntrospectionException e) {
			throw new RuntimeException(e);
		}
	}

	private static boolean validate(Object bean,
			PropertyDescriptor descriptor,
			ActionMessages errors, HttpServletRequest request) {

		Method writeMethod = descriptor.getWriteMethod();
		Method readMethod = descriptor.getReadMethod();
		Object value = invoke(readMethod, bean);

		Validators validators = writeMethod
				.getAnnotation(Validators.class);

		if (value == null
				|| StringUtil.isEmpty(value.toString())) {
			return true;
		}
		if (validators == null) {
			return true;
		}

		Validator[] validatorDefs = validators.value();
		PropertyName propertyName = writeMethod
				.getAnnotation(PropertyName.class);

		for (int i = 0; i < validatorDefs.length; i++) {
			Validator validatorDef = validatorDefs[i];
			Class<? extends JValidator> clazz = validatorDef
					.value();
			JValidator jValidator;
			try {
				jValidator = clazz.newInstance();
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
			Property[] properties = validatorDef.properties();
			setProperties(jValidator, properties);
			jValidator.setBean(bean);
			jValidator.setRequest(request);
			String property = descriptor.getName();
			jValidator.setPropertyName(propertyName.value());
			try {
				boolean valid = jValidator.validate(value
						.toString());
				if (valid == false) {
					throw new IllegalStateException();
				}
			} catch (ValidateException e) {
				addMessage(property, e, errors);
				return false;
			}
		}
		return true;

	}

	private static void addMessage(String property,
			ValidateException e, ActionMessages errors) {

		ActionMessages messages = new ActionMessages();
		ActionMessage message = new ActionMessage(e
				.getMessageKey(), e.getMessageArgs());
		messages.add(property, message);
		errors.add(messages);
	}

	private static void setProperties(JValidator validator,
			Property[] properties) {

		if (properties == null) {
			return;
		}

		BeanInfo beanInfo = getBeanInfo(validator);
		for (int i = 0; i < properties.length; i++) {
			Property property = properties[i];
			PropertyDescriptor propertyDesc = getPropertyDescriptor(
					beanInfo, property.name());
			Method writeMethod = propertyDesc.getWriteMethod();
			invoke(writeMethod, validator, property.value());
		}
	}

	private static Object invoke(Method method, Object target,
			Object... args) {

		try {
			return method.invoke(target, args);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	// TODO cache
	private static PropertyDescriptor getPropertyDescriptor(
			BeanInfo beanInfo, String propertyName) {

		PropertyDescriptor[] propertyDescriptors = beanInfo
				.getPropertyDescriptors();
		for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
			if (propertyDescriptor.getName()
					.equals(propertyName) == true) {
				return propertyDescriptor;
			}
		}
		// TODO set error message
		throw new IllegalArgumentException();
	}
}
