package jp.sbcr.java_framework.validator.validators;

import javax.servlet.http.HttpServletRequest;

import jp.sbcr.java_framework.validator.JValidator;
import jp.sbcr.java_framework.validator.exception.ValidateException;

/**
 * {@link JValidator#validate(String)}に対してのテンプレートメソッドと、
 * Beanとリクエストのアクセサメソッドを提供します。
 */
public abstract class AbstractValidator implements JValidator {

	private Object bean;

	private HttpServletRequest request;

	private String propertyName;

	public abstract boolean validateNoException(String value);

	/**
	 * <p>
	 * {@link JValidator#validateNoException(String)}がfalseの場合に、
	 * {@link #getMessageResourceKey()}で取得できるメッセージキーを使用して、
	 * ValidateExceptionをスローします。
	 * </p>
	 * <p>
	 * メッセージのフォーマット時に、置換オブジェクトを使用する場合は、オーバーライドする必要があります
	 * </p>
	 */
	public boolean validate(String value)
			throws ValidateException {
		boolean isValid = validateNoException(value);
		if (isValid == false) {
			throw new ValidateException(getMessageResourceKey(),
					propertyName);
		}
		return true;
	}

	protected abstract String getMessageResourceKey();

	public Object getBean() {
		return bean;
	}

	public void setBean(Object bean) {
		this.bean = bean;
	}

	public HttpServletRequest getRequest() {
		return request;
	}

	public void setRequest(HttpServletRequest request) {
		this.request = request;
	}

	public String getPropertyName() {
		return propertyName;
	}

	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

}
